import copy

from django.core.exceptions import ValidationError
from django.utils.datastructures import SortedDict

from rest_framework import serializers, fields, relations

from arangodb.orm.fields import NumberField, CharField, ForeignKeyField, ManyToManyField, BooleanField, ChoiceField, \
    DateField, DatetimeField

from djara.django.models.fields import FileField
from djara.django.restframework.fields import RestFileField
from djara.django.restframework.relations import RelatedCollectionModelField


class CollectionModelSerializerOptions(serializers.SerializerOptions):
    """
    Meta class options for CollectionModelSerializer
    """
    def __init__(self, meta):
        super(CollectionModelSerializerOptions, self).__init__(meta)
        self.model = getattr(meta, 'model', None)
        self.read_only_fields = getattr(meta, 'read_only_fields', ())
        self.write_only_fields = getattr(meta, 'write_only_fields', ())

class CollectionModelSerializer(serializers.Serializer):
    """
    """

    _options_class = CollectionModelSerializerOptions

    field_mapping = {

        # ArangoPy fields
        BooleanField: fields.BooleanField,
        NumberField: fields.IntegerField,
        CharField: fields.CharField,
        ChoiceField: fields.ChoiceField,
        DateField: fields.DateField,
        DatetimeField: fields.DateTimeField,

        # Only Django fields
        FileField: RestFileField,
    }

    def get_fields(self):
        """
        """

        return_fields = SortedDict()

        # Get the explicitly declared fields
        base_fields = copy.deepcopy(self.base_fields)
        for key, field in base_fields.items():
            return_fields[key] = field

        # Add in the default fields
        default_fields = self.get_default_fields()
        for key, val in default_fields.items():
            if key not in return_fields:
                return_fields[key] = val

        model_class = self.opts.model
        displayed_fields = self.opts.fields

        model_class_fields = model_class.get_collection_fields_dict()

        if 'id' in displayed_fields:
            return_fields['id'] = fields.CharField(read_only=True)

        if 'key' in displayed_fields:
            return_fields['key'] = fields.IntegerField(read_only=True)

        for field_name in displayed_fields:

            if field_name == 'id' or field_name == 'key':
                continue

            if field_name in return_fields:
                continue

            field = model_class_fields[field_name]

            kwargs = {}

            if field.__class__ in self.field_mapping:
                restframework_field_class = self.field_mapping[field.__class__]
                restframework_field = restframework_field_class(**kwargs)

                return_fields[field_name] = restframework_field

            # get_related_field
            if isinstance(field, ForeignKeyField):
                return_fields[field_name] = self.get_related_field(model_field=field, related_model=field.relation_class, to_many=False)
            if isinstance(field, ManyToManyField):
                return_fields[field_name] = self.get_related_field(model_field=field, related_model=field.relation_class, to_many=True)

        return return_fields

    def get_field(self, model_field):
        """
        Creates a default instance of a basic non-relational field.
        """

    def get_related_field(self, model_field, related_model, to_many):
        """
        Creates a default instance of a flat relational field.

        Note that model_field will be `None` for reverse relationships.
        """
        # TODO: filter queryset using:
        # .using(db).complex_filter(self.rel.limit_choices_to)

        kwargs = {
            'queryset': related_model.objects,
            'many': to_many
        }

        if model_field:
            kwargs['required'] = not(model_field.null or model_field.blank)
            # if model_field.help_text is not None:
            #     kwargs['help_text'] = model_field.help_text
            # if model_field.verbose_name is not None:
            #     kwargs['label'] = model_field.verbose_name

            # if not model_field.editable:
            #     kwargs['read_only'] = True

            # if model_field.verbose_name is not None:
            #     kwargs['label'] = model_field.verbose_name

            # if model_field.help_text is not None:
            #     kwargs['help_text'] = model_field.help_text

        return RelatedCollectionModelField(**kwargs)

    def restore_fields(self, data, files):
        """
        Core of deserialization, together with `restore_object`.
        Converts a dictionary of data into a dictionary of deserialized fields.
        """
        reverted_data = {}

        if isinstance(data, basestring):
            model = self.opts.model.objects.get(_id=data)
            self.object = model
            return model

        if data is not None and not isinstance(data, dict):
            self._errors['non_field_errors'] = ['Invalid data']
            return None

        for field_name, field in self.fields.items():
            field.initialize(parent=self, field_name=field_name)
            try:
                field.field_from_native(data, files, field_name, reverted_data)
            except ValidationError as err:
                self._errors[field_name] = list(err.messages)

        return reverted_data

    def restore_object(self, attrs, instance=None):
        """
        Deserialize a dictionary of attributes into an object instance.
        You should override this method to control how deserialized objects
        are instantiated.
        """

        if instance is None:
            instance = self.opts.model()
        else:
            return instance

        if instance is not None:
            for attribute_name in attrs:
                attribute_value = attrs[attribute_name]
                setattr(instance, attribute_name, attribute_value)
            return instance
        return attrs

    def save_object(self, obj, **kwargs):

        model_class = self.opts.model
        obj = model_class.objects._create_model_from_dict(obj)
        self.object = obj

        obj.save()

    def delete_object(self, obj):
        obj.delete()

    def save(self, **kwargs):
        """
        Save the deserialized object and return it.
        """
        # Clear cached _data, which may be invalidated by `save()`
        self._data = None

        if isinstance(self.object, list):
            [self.save_object(item, **kwargs) for item in self.object]

            if self.object._deleted:
                [self.delete_object(item) for item in self.object._deleted]
        else:
            self.save_object(self.object, **kwargs)

        return self.object


from django import template
from django.template import loader, Context
from django.template.base import Template

from djara.django.menus.models import GlobalMenu

register = template.Library()


def show_global_menu(context, menu_template='djara/django/menus/simple_menu.html'):
    """
    """

    request = context['request']

    template_context = Context({
        'menu_items': GlobalMenu.instance().items,
        'current_view': request.resolver_match.view_name,
        'user': context['user'],
    })

    template = loader.get_template(menu_template)

    rendered = template.render(template_context)

    return rendered

register.simple_tag(takes_context=True)(show_global_menu)


def show_active_sub_menu(context, menu_template='djara/django/menus/simple_menu.html'):
    """
    """

    request = context['request']
    current_view = request.resolver_match.view_name

    def get_active_menu(menu_tree):
        active_item = None

        for item in menu_tree.items:
            # Check if this item is the current view
            if item.is_current_item(current_view):
                active_item = item
                break

            # Check if a child of the item is the curren view
            child_item = item.get_child(current_view)
            if child_item is not None:
                child_item.active = True
                active_item = item
                break

        return active_item

    active_menu_item = get_active_menu(GlobalMenu.instance())

    return render_menu_children(context, active_menu_item, menu_template)

register.simple_tag(takes_context=True)(show_active_sub_menu)


def render_menu_children(context, active_menu_item, menu_template='djara/django/menus/simple_menu.html'):
    """
    """

    request = context['request']

    if active_menu_item is None:
        children = []
    else:
        children = active_menu_item.children

    template_context = Context({
        'menu_items': children,
        'current_view': request.resolver_match.view_name,
        'user': context['user'],
    })

    template = loader.get_template(menu_template)

    rendered = template.render(template_context)

    return rendered


class ShowPermittedMenuEntryNode(template.Node):

    def __init__(self, nodelist_empty, menu_item_variable):
        self.nodelist_empty = nodelist_empty
        self.menu_item_variable = menu_item_variable

    def render(self, context):
        ''' If a variable is passed, it is found as string, so first we use render to convert that
            variable to a string, and then we render that string. '''
        if self.nodelist_empty is None:
            return ''
        else:
            user_variable = template.Variable('user').resolve(context)
            resolved_menu_item = template.Variable(self.menu_item_variable).resolve(context)

            if resolved_menu_item.permission_method(user_variable) is True:
                template_string = ''

                # Set if is current view
                current_view = context['current_view']

                # Check if it is the current view
                if resolved_menu_item.is_current_item(current_view):
                    resolved_menu_item.active = True

                # Check if a child of it is active
                child_view = resolved_menu_item.get_child(current_view)
                if child_view is not None:
                    resolved_menu_item.active = True

                # Render template
                for node in self.nodelist_empty:
                    template_string += node.render(context)
                rendered = Template(template_string).render(context)

                # Unset current view
                resolved_menu_item.active = False

                return rendered
            else:
                return ''


def show_permitted_menu_entry(parser, token):
    """
    """

    try:
        menu_item_variable = token.split_contents()[1]
        nodelist_loop = parser.parse(('empty', 'endshow_permitted_menu_entry',))
        token = parser.next_token()
        if token.contents == 'empty':
            nodelist_empty = parser.parse(('endshow_permitted_menu_entry',))
            parser.delete_first_token()
        else:
            nodelist_empty = None
    except Exception:
        raise template.TemplateSyntaxError('Syntax for show_permitted_menu_entry tag is wrong.')
    return ShowPermittedMenuEntryNode(nodelist_loop, menu_item_variable)

register.tag('show_permitted_menu_entry', show_permitted_menu_entry)


from djara.django.contrib import admin
from djara.django.contrib.admin.options import CollectionAdmin
from djara.django.auth.models import User

""" models """
class UserAdmin(CollectionAdmin):
    """
    User Admin
    """
    list_display = ('username', 'email_address','is_staff_member', 'is_owner', 'is_active', 'is_logged_in', )
    list_filter = ('is_staff_member', 'is_owner', 'is_active', 'is_logged_in', )
    exclude = ( 'groups', 'permissions', )

admin.site.register(User, UserAdmin)


from django import forms
from django.contrib.admin.options import ModelAdmin, IS_POPUP_VAR
from django.contrib.admin.templatetags.admin_urls import add_preserved_filters
from django.contrib.admin.util import flatten_fieldsets
from django.contrib.admin.views.main import TO_FIELD_VAR
from django.core.exceptions import FieldError
from django.core.urlresolvers import reverse
from django.forms.models import modelform_defines_fields, modelform_factory
from django.template.response import TemplateResponse
from djara.django.forms.forms import CollectionForm


class CollectionAdmin(ModelAdmin):

    def __init__(self, model, admin_site):

        super(CollectionAdmin, self).__init__(model, admin_site)


    form = CollectionForm

    view_on_site = True

    def get_form(self, request, obj=None, **kwargs):
        """
        Returns a Form class for use in the admin add view. This is used by
        add_view and change_view.
        """
        if 'fields' in kwargs:
            fields = kwargs.pop('fields')
        else:
            fields = flatten_fieldsets(self.get_fieldsets(request, obj))
        if self.exclude is None:
            exclude = []
        else:
            exclude = list(self.exclude)
        exclude.extend(self.get_readonly_fields(request, obj))
        if self.exclude is None and hasattr(self.form, '_meta') and self.form._meta.exclude:
            # Take the custom ModelForm's Meta.exclude into account only if the
            # ModelAdmin doesn't define its own.
            exclude.extend(self.form._meta.exclude)
        # if exclude is an empty list we pass None to be consistent with the
        # default on modelform_factory
        exclude = exclude or None
        defaults = {
            "form": self.form,
            "fields": fields,
            "exclude": exclude,
        }
        defaults.update(kwargs)

        if defaults['fields'] is None and not modelform_defines_fields(defaults['form']):
            defaults['fields'] = forms.ALL_FIELDS

        try:
            return modelform_factory(self.model, **defaults)
        except FieldError as e:
            raise FieldError('%s. Check fields/fieldsets/exclude attributes of class %s.'
                             % (e, self.__class__.__name__))
        

    def get_view_on_site_url(self, obj=None):
        if obj is None or not self.view_on_site:
            return None

        if callable(self.view_on_site):
            return self.view_on_site(obj)
        
        elif self.view_on_site and hasattr(obj, 'get_absolute_url'):
            # use the ContentType lookup if view_on_site is True
            return reverse('admin:view_on_site', kwargs={
                'content_type_id': obj.collection_name,
                'object_id': obj.id
            })
        

    def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
        opts = self.model._meta
        app_label = opts.app_label
        preserved_filters = self.get_preserved_filters(request)
        form_url = add_preserved_filters({'preserved_filters': preserved_filters, 'opts': opts}, form_url)
        view_on_site_url = self.get_view_on_site_url(obj)
        context.update({
            'add': add,
            'change': change,
            'has_add_permission': self.has_add_permission(request),
            'has_change_permission': self.has_change_permission(request, obj),
            'has_delete_permission': self.has_delete_permission(request, obj),
            'has_file_field': True,  # FIXME - this should check if form or formsets have a FileField,
            'has_absolute_url': view_on_site_url is not None,
            'absolute_url': view_on_site_url,
            'form_url': form_url,
            'opts': opts,
            'content_type_id': self.model.collection_instance.name,
            'save_as': self.save_as,
            'save_on_top': self.save_on_top,
            'to_field_var': TO_FIELD_VAR,
            'is_popup_var': IS_POPUP_VAR,
            'app_label': app_label,
        })
        if add and self.add_form_template is not None:
            form_template = self.add_form_template
        else:
            form_template = self.change_form_template

        return TemplateResponse(request, form_template or [
            "admin/%s/%s/change_form.html" % (app_label, opts.model_name),
            "admin/%s/change_form.html" % app_label,
            "admin/change_form.html"
        ], context, current_app=self.admin_site.name)


    def get_queryset(self, request):
        """
        Returns a QuerySet of all model instances that can be edited by the
        admin site. This is used by changelist_view.
        """
        qs = self.model.objects.all()
        # TODO: this should be handled by some parameter to the ChangeList.
        ordering = self.get_ordering(request)
        if ordering:
            qs = qs.order_by(*ordering)
        return qs


from rest_framework.routers import DefaultRouter



class CollectionModelRouter(DefaultRouter):

    def get_default_base_name(self, viewset):
        """
        If `base_name` is not specified, attempt to automatically determine
        it from the viewset.
        """
        # Note that `.model` attribute on views is deprecated, although we
        # enforce the deprecation on the view `get_serializer_class()` and
        # `get_queryset()` methods, rather than here.
        model_cls = getattr(viewset, 'model', None)

        if model_cls:
            return model_cls.__name__.lower()


from django.conf import settings
from django.core.urlresolvers import reverse_lazy
from django.http.response import HttpResponseNotFound
from django.shortcuts import render, redirect
from django.template import loader
from django.utils.crypto import get_random_string
from django.utils.translation import ugettext as _

from djara.django.auth.models import User
from djara.django.auth.utils import login, logout


def login_view(request):
    """
    """

    return render(request, 'auth/user/login.html', {})


def login_action(request, view_name):
    """
    """

    username = request.POST.get('username', None)
    user_password = request.POST.get('user_password', None)
    
    user = User.objects.get(username=username)
    # User login
    if user and user.login(password=user_password):

        # Django session login
        login(request=request, user=user)
        return redirect(to=view_name)
    else:
        return render(request, 'auth/user/login.html', {})


def logout_action(request, view_name):
    """
    """

    # User logout
    request.user.logout()

    # Django logout
    if logout(request):
        return redirect(to=view_name)


def register_view(request):
    """
    """

    return render(request, 'auth/user/register.html', {})


def register_action(request, view_name):
    """
    """

    username = request.POST.get('username', None)
    email = request.POST.get('email', None)
    user_password = request.POST.get('user_password', None)
    user_password_again = request.POST.get('user_password_again', None)

    # Check username and email
    if User.objects.get(username=username) or User.objects.get(email_address=email):
        raise Exception('Username or email is already taken')

    # Check passwords
    if user_password != user_password_again:
        raise Exception('Passwords are not the same')

    activation_key = get_random_string(length=20)

    user = User()
    user.username = username
    user.email_address = email
    user.password = user_password
    user.activation_key = activation_key
    user.save()

    kwargs = {
        'activation_key': activation_key,
        'uuid': user.uuid,
        'view_name': view_name,
    }

    # register_msg
    register_template_string = loader.render_to_string('auth/user/register_email.html', dictionary={
        'logo_url': '%s%simages/logo.png' % (settings.BASE_URL, settings.STATIC_URL),
        'activate_account_url': '%s%s' % (settings.BASE_URL, reverse_lazy('auth:register-activate', kwargs=kwargs)),
    })

    user.email_user(subject=_('Registration'), message=register_template_string, from_email=settings.REGISTRATION_EMAIL)

    return render(request, 'auth/user/register_successful.html', {})


def register_activate(request, activation_key, uuid, view_name):
    """
    """

    user = User.objects.get(uuid=uuid, activation_key=activation_key)
    if not user or not activation_key or len(activation_key) < 5:
        return HttpResponseNotFound()

    user.is_active = True
    user.activation_key = ''
    user.save()

    return render(request, 'auth/user/register_finished.html', {})


from django.utils.functional import SimpleLazyObject

from djara.django.auth.utils import get_session_user


def get_user(request):
    if not hasattr(request, '_cached_user'):
        request._cached_user = get_session_user(request)
    return request._cached_user


class AuthenticationMiddleware(object):
    def process_request(self, request):
        assert hasattr(request, 'session'), "The Django authentication middleware requires session middleware to be installed. Edit your MIDDLEWARE_CLASSES setting to insert 'django.contrib.sessions.middleware.SessionMiddleware'."

        request.user = SimpleLazyObject(lambda: get_user(request))



from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, QueryDict
from django.template.response import TemplateResponse
from django.utils.http import base36_to_int, is_safe_url, urlsafe_base64_decode, urlsafe_base64_encode
from django.utils.translation import ugettext as _
from django.utils.six.moves.urllib.parse import urlparse, urlunparse
from django.shortcuts import resolve_url
from django.utils.encoding import force_bytes, force_text
from django.views.decorators.debug import sensitive_post_parameters
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect

# Avoid shadowing the login() and logout() views below.
from django.contrib.auth import REDIRECT_FIELD_NAME, logout as auth_logout, get_user_model
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import AuthenticationForm, PasswordResetForm, SetPasswordForm, PasswordChangeForm
from django.contrib.auth.tokens import default_token_generator
from django.contrib.sites.models import get_current_site

from djara.django.contrib.admin.forms import CollectionAuthenticationForm
from djara.django.auth.utils import login as auth_login


@sensitive_post_parameters()
@csrf_protect
@never_cache
def login(request, template_name='registration/login.html',
          redirect_field_name=REDIRECT_FIELD_NAME,
          authentication_form=CollectionAuthenticationForm,
          current_app=None, extra_context=None):
    """
    Displays the login form and handles the login action.
    """
    redirect_to = request.REQUEST.get(redirect_field_name, '')

    if request.method == "POST":
        form = authentication_form(request, data=request.POST)
        if form.is_valid():

            # Ensure the user-originating redirection url is safe.
            if not is_safe_url(url=redirect_to, host=request.get_host()):
                redirect_to = resolve_url(settings.LOGIN_REDIRECT_URL)

            # Okay, security check complete. Log the user in.
            auth_login(request, form.get_user())

            return HttpResponseRedirect(redirect_to)
    else:
        form = authentication_form(request)

    current_site = get_current_site(request)

    context = {
        'form': form,
        redirect_field_name: redirect_to,
        'site': current_site,
        'site_name': current_site.name,
    }
    if extra_context is not None:
        context.update(extra_context)
    return TemplateResponse(request, template_name, context,
                            current_app=current_app)


def logout(request, next_page=None,
           template_name='registration/logged_out.html',
           redirect_field_name=REDIRECT_FIELD_NAME,
           current_app=None, extra_context=None):
    """
    Logs out the user and displays 'You are logged out' message.
    """
    auth_logout(request)

    if next_page is not None:
        next_page = resolve_url(next_page)

    if redirect_field_name in request.REQUEST:
        next_page = request.REQUEST[redirect_field_name]
        # Security check -- don't allow redirection to a different host.
        if not is_safe_url(url=next_page, host=request.get_host()):
            next_page = request.path

    if next_page:
        # Redirect to this page until the session has been cleared.
        return HttpResponseRedirect(next_page)

    current_site = get_current_site(request)
    context = {
        'site': current_site,
        'site_name': current_site.name,
        'title': _('Logged out')
    }
    if extra_context is not None:
        context.update(extra_context)
    return TemplateResponse(request, template_name, context,
        current_app=current_app)


def logout_then_login(request, login_url=None, current_app=None, extra_context=None):
    """
    Logs out the user if he is logged in. Then redirects to the log-in page.
    """
    if not login_url:
        login_url = settings.LOGIN_URL
    login_url = resolve_url(login_url)
    return logout(request, login_url, current_app=current_app, extra_context=extra_context)


def redirect_to_login(next, login_url=None,
                      redirect_field_name=REDIRECT_FIELD_NAME):
    """
    Redirects the user to the login page, passing the given 'next' page
    """
    resolved_url = resolve_url(login_url or settings.LOGIN_URL)

    login_url_parts = list(urlparse(resolved_url))
    if redirect_field_name:
        querystring = QueryDict(login_url_parts[4], mutable=True)
        querystring[redirect_field_name] = next
        login_url_parts[4] = querystring.urlencode(safe='/')

    return HttpResponseRedirect(urlunparse(login_url_parts))


# 4 views for password reset:
# - password_reset sends the mail
# - password_reset_done shows a success message for the above
# - password_reset_confirm checks the link the user clicked and
#   prompts for a new password
# - password_reset_complete shows a success message for the above

# @csrf_protect
# def password_reset(request, is_admin_site=False,
#                    template_name='registration/password_reset_form.html',
#                    email_template_name='registration/password_reset_email.html',
#                    subject_template_name='registration/password_reset_subject.txt',
#                    password_reset_form=PasswordResetForm,
#                    token_generator=default_token_generator,
#                    post_reset_redirect=None,
#                    from_email=None,
#                    current_app=None,
#                    extra_context=None):
#     if post_reset_redirect is None:
#         post_reset_redirect = reverse('password_reset_done')
#     else:
#         post_reset_redirect = resolve_url(post_reset_redirect)
#     if request.method == "POST":
#         form = password_reset_form(request.POST)
#         if form.is_valid():
#             opts = {
#                 'use_https': request.is_secure(),
#                 'token_generator': token_generator,
#                 'from_email': from_email,
#                 'email_template_name': email_template_name,
#                 'subject_template_name': subject_template_name,
#                 'request': request,
#             }
#             if is_admin_site:
#                 opts = dict(opts, domain_override=request.get_host())
#             form.save(**opts)
#             return HttpResponseRedirect(post_reset_redirect)
#     else:
#         form = password_reset_form()
#     context = {
#         'form': form,
#     }
#     if extra_context is not None:
#         context.update(extra_context)
#     return TemplateResponse(request, template_name, context,
#                             current_app=current_app)
#
#
# def password_reset_done(request,
#                         template_name='registration/password_reset_done.html',
#                         current_app=None, extra_context=None):
#     context = {}
#     if extra_context is not None:
#         context.update(extra_context)
#     return TemplateResponse(request, template_name, context,
#                             current_app=current_app)
#
#
# # Doesn't need csrf_protect since no-one can guess the URL
# @sensitive_post_parameters()
# @never_cache
# def password_reset_confirm(request, uidb64=None, token=None,
#                            template_name='registration/password_reset_confirm.html',
#                            token_generator=default_token_generator,
#                            set_password_form=SetPasswordForm,
#                            post_reset_redirect=None,
#                            current_app=None, extra_context=None):
#     """
#     View that checks the hash in a password reset link and presents a
#     form for entering a new password.
#     """
#     UserModel = get_user_model()
#     assert uidb64 is not None and token is not None  # checked by URLconf
#     if post_reset_redirect is None:
#         post_reset_redirect = reverse('password_reset_complete')
#     else:
#         post_reset_redirect = resolve_url(post_reset_redirect)
#     try:
#         uid = urlsafe_base64_decode(uidb64)
#         user = UserModel._default_manager.get(pk=uid)
#     except (TypeError, ValueError, OverflowError, UserModel.DoesNotExist):
#         user = None
#
#     if user is not None and token_generator.check_token(user, token):
#         validlink = True
#         if request.method == 'POST':
#             form = set_password_form(user, request.POST)
#             if form.is_valid():
#                 form.save()
#                 return HttpResponseRedirect(post_reset_redirect)
#         else:
#             form = set_password_form(None)
#     else:
#         validlink = False
#         form = None
#     context = {
#         'form': form,
#         'validlink': validlink,
#     }
#     if extra_context is not None:
#         context.update(extra_context)
#     return TemplateResponse(request, template_name, context,
#                             current_app=current_app)
#
# def password_reset_confirm_uidb36(request, uidb36=None, **kwargs):
#     # Support old password reset URLs that used base36 encoded user IDs.
#     # Remove in Django 1.7
#     try:
#       uidb64 = force_text(urlsafe_base64_encode(force_bytes(base36_to_int(uidb36))))
#     except ValueError:
#       uidb64 = '1' # dummy invalid ID (incorrect padding for base64)
#     return password_reset_confirm(request, uidb64=uidb64, **kwargs)
#
# def password_reset_complete(request,
#                             template_name='registration/password_reset_complete.html',
#                             current_app=None, extra_context=None):
#     context = {
#         'login_url': resolve_url(settings.LOGIN_URL)
#     }
#     if extra_context is not None:
#         context.update(extra_context)
#     return TemplateResponse(request, template_name, context,
#                             current_app=current_app)
#
#
# @sensitive_post_parameters()
# @csrf_protect
# @login_required
# def password_change(request,
#                     template_name='registration/password_change_form.html',
#                     post_change_redirect=None,
#                     password_change_form=PasswordChangeForm,
#                     current_app=None, extra_context=None):
#     if post_change_redirect is None:
#         post_change_redirect = reverse('password_change_done')
#     else:
#         post_change_redirect = resolve_url(post_change_redirect)
#     if request.method == "POST":
#         form = password_change_form(user=request.user, data=request.POST)
#         if form.is_valid():
#             form.save()
#             return HttpResponseRedirect(post_change_redirect)
#     else:
#         form = password_change_form(user=request.user)
#     context = {
#         'form': form,
#     }
#     if extra_context is not None:
#         context.update(extra_context)
#     return TemplateResponse(request, template_name, context,
#                             current_app=current_app)
#
#
# @login_required
# def password_change_done(request,
#                          template_name='registration/password_change_done.html',
#                          current_app=None, extra_context=None):
#     context = {}
#     if extra_context is not None:
#         context.update(extra_context)
#     return TemplateResponse(request, template_name, context,
#                             current_app=current_app)



from django import forms
from django.contrib.auth.forms import AuthenticationForm

from djara.django.auth.utils import authenticate


class CollectionAuthenticationForm(AuthenticationForm):


    def clean(self):
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')

        if username and password:

            self.user_cache = authenticate(username=username, password=password)

            # self.user_cache = authenticate(username=username,
            #                                password=password)
            if self.user_cache is None:
                raise forms.ValidationError(
                    self.error_messages['invalid_login'],
                    code='invalid_login',
                    params={'username': self.username_field.verbose_name},
                )
            elif not self.user_cache.is_active:
                raise forms.ValidationError(
                    self.error_messages['inactive'],
                    code='inactive',
                )
        return self.cleaned_data


class MenuTree(object):

    def __init__(self):
        """
        """

        self.items = []

    def get_item(self, key):
        """
        """

        for item_obj in self.items:
            if item_obj.name == key or item_obj.view == key:
                return item_obj

        raise Exception('No item found')


class MenuTreeItem(object):

    def __init__(self, key, display_name, view, permission_method, **kwargs):
        """
        """

        # To which view it is mapped
        self.view = view

        self.key = key
        self.display_name = display_name

        self.items = []

        self.permission_method = permission_method

        # Temporary variable
        self.active = False

        # Children
        self.children = kwargs.pop('children', [])

    def add_child(self, child):
        """
        """

        self.children.append(child)

    def get_child(self, view_name):
        """
        """

        child_with_view = None

        for child in self.children:
            if child.view == view_name:
                child_with_view = child
                break

        return child_with_view

    def has_child(self, view_name):
        """
        """

        return self.get_child(view_name) is not None


    def is_current_item(self, current_view):
        """
        """

        return self.view == current_view


class GlobalMenu(MenuTree):

    _instance = None

    def __init__(self):
        super(GlobalMenu, self).__init__()

    @classmethod
    def init(cls):
        """
        """

        if cls._instance is None:
            cls._instance = cls()

    @classmethod
    def instance(cls):
        """
        """

        cls.init()
        return cls._instance

    @classmethod
    def add_menu_item(cls, item, parent_key=None):
        """
        """

        if parent_key is None:
            cls.instance().items.append(item)
        else:
            items = cls.instance().items

            # Iterate over all items and search the parent
            for parent_item in items:
                if parent_item.key == parent_key:
                    parent_item.add_child(item)
                    break

    @classmethod
    def item(cls, key):
        """
        """

        return cls.instance().get_item(key=key)


from django.utils import timezone
from django.utils.translation import ugettext as _

from arangodb.index.unique import HashIndex
from arangodb.orm.fields import CharField, BooleanField, DatetimeField, UuidField, ManyToManyField

from djara.django.auth.fields import PasswordField
from djara.django.mail.utils import send_html_email
from djara.django.models.fields import DjangoBooleanField, DjangoTimeField
from djara.django.models.model import DjangoModel

# extend_meta_data
class BaseModel(DjangoModel):

    uuid = UuidField()


class PermissionMixin(object):

    def has_permission(self, name):
        pass


class Permission(DjangoModel):

    collection_name = 'djara_permission'

    name = CharField(null=False)


class Group(DjangoModel, PermissionMixin):

    collection_name = 'djara_group'

    permissions = ManyToManyField(to=Permission, related_name='groups')


class User(BaseModel, PermissionMixin):

    collection_name = 'djara_user'

    username_index = HashIndex(fields=['username'])
    email_address_index = HashIndex(fields=['email_address'])

    username = CharField(verbose_name=_('Username'), null=False)
    email_address = CharField(verbose_name=_('Mail address'), max_length=2048, null=False)

    password = PasswordField()

    activation_key = CharField(verbose_name=_('Activation key'), null=True, default=None)
    is_active = DjangoBooleanField(verbose_name=_('Is active'), default=False, null=False)

    #
    is_staff_member = DjangoBooleanField(verbose_name=_('Is staff'), default=False, null=False)
    is_owner = DjangoBooleanField(verbose_name=_('Is owner'), default=False, null=False)

    # Login status
    is_logged_in = DjangoBooleanField(verbose_name=_('Is logged in'), default=False, null=False)
    last_login_time = DjangoTimeField(verbose_name=_('Last login'))

    groups = ManyToManyField(to=Group, related_name='users')
    permissions = ManyToManyField(to=Permission, related_name='users')


    # Helper methods
    def login(self, password):
        """
        """

        if not self.is_active:
            raise Exception('Not active')

        if not self.password.verify(password=password):
            raise Exception('Wrong password')

        self.is_logged_in = True
        self.last_login_time = timezone.now()
        self.save()

        return True

    def logout(self):
        """
        """

        if not self.is_active:
            raise Exception('Not active')

        if not self.is_logged_in:
            raise Exception('Not logged in')

        self.is_logged_in = False
        self.save()

    def is_authenticated(self):
        """
        """

        return True

    def is_staff(self):
        """
        """

        return self.is_staff_member

    def has_module_perms(self, app_label):
        """
        """

        return self.is_staff()

    def has_perm(self, code):
        """
        """

        return self.is_staff()

    def email_user(self, subject, message, from_email=None):
        """
        Sends an email to this User.
        """
        send_html_email(subject, message, from_email, [self.email_address])


    def __unicode__(self):
        return u'%s <%s>' % (self.username, self.email_address)


    class Meta(object):
        app_label = 'auth'
        verbose_name = 'User'
        verbose_name_plural = 'Users'
        ordering = ('username', )


class AnonymousUser(User, PermissionMixin):

    def is_authenticated(self):
        return False


from arangodb.orm.fields import CharField
from django.contrib.auth.hashers import PBKDF2SHA1PasswordHasher

hasher = PBKDF2SHA1PasswordHasher()


class PasswordField(CharField):

    def get(self):
        """
        """

        return self

    def on_save(self, model_instance):
        """
        """

        # Check if the password is not already hashed
        if not '$' in self.text or not self.text.count('$') == 3 or hasher.must_update(self.text):
            salt = hasher.salt()
            self.text = '%s' % ( hasher.encode(password=self.text, salt=salt, iterations=12000) )

    def verify(self, password):
        hashed_password = self.text
        return hasher.verify(password=password, encoded=hashed_password)


import django
from django.contrib.admin.sites import AdminSite as DjangoAdminSite
from django.core.urlresolvers import reverse, NoReverseMatch
from django.http.response import Http404
from django.template.defaultfilters import capfirst
from django.template.response import TemplateResponse
from django.views.decorators.cache import never_cache
from django.utils.translation import ugettext as _
import six


class CollectionAdminSite(DjangoAdminSite):


    def register(self, model_or_iterable, admin_class=None, **options):
        """
        Registers the given model(s) with the given admin class.

        The model(s) should be Model classes, not instances.

        If an admin class isn't given, it will use ModelAdmin (the default
        admin options). If keyword arguments are given -- e.g., list_display --
        they'll be applied as options to the admin class.

        If a model is already registered, this will raise AlreadyRegistered.

        If a model is abstract, this will raise ImproperlyConfigured.
        """
        # if not admin_class:
        #     admin_class = ModelAdmin

        if not isinstance(model_or_iterable, list):
            model_or_iterable = [model_or_iterable]
        for model in model_or_iterable:
        #     if model._meta.abstract:
        #         raise ImproperlyConfigured('The model %s is abstract, so it '
        #               'cannot be registered with admin.' % model.__name__)
        #
        #     if model in self._registry:
        #         raise AlreadyRegistered('The model %s is already registered' % model.__name__)
        #
        #     # Ignore the registration if the model has been
        #     # swapped out.
        #     if not model._meta.swapped:
        #         # If we got **options then dynamically construct a subclass of
        #         # admin_class with those **options.
        #         if options:
        #             # For reasons I don't quite understand, without a __module__
        #             # the created class appears to "live" in the wrong place,
        #             # which causes issues later on.
        #             options['__module__'] = __name__
        #             admin_class = type("%sAdmin" % model.__name__, (admin_class,), options)
        #
        #         if admin_class is not ModelAdmin and settings.DEBUG:
        #             admin_class.validate(model)
        #
        #         # Instantiate the admin class to save in the registry
            self._registry[model] = admin_class(model, self)

    @never_cache
    def index(self, request, extra_context=None):
        """
        Displays the main admin index page, which lists all of the installed
        apps that have been registered in this site.
        """
        app_dict = {}
        user = request.user
        for model, model_admin in self._registry.items():
            app_label = model._meta.app_label
            has_module_perms = user.has_module_perms(app_label)

            if has_module_perms:
                perms = model_admin.get_model_perms(request)

                # Check whether user has any perm for this module.
                # If so, add the module to the model_list.
                if True in perms.values():
                    info = (app_label, model._meta.model_name)
                    model_dict = {
                        'name': capfirst(model._meta.verbose_name_plural),
                        'object_name': model._meta.object_name,
                        'perms': perms,
                    }
                    if perms.get('change', False):
                        try:
                            model_dict['admin_url'] = reverse('admin:admin:%s_%s_changelist' % info, current_app=self.name)
                        except NoReverseMatch:
                            pass
                    if perms.get('add', False):
                        try:
                            model_dict['add_url'] = reverse('admin:admin:%s_%s_add' % info, current_app=self.name)
                        except NoReverseMatch:
                            pass
                    if app_label in app_dict:
                        print(model_dict)
                        app_dict[app_label]['models'].append(model_dict)
                    else:
                        app_dict[app_label] = {
                            'name': app_label.title(),
                            'app_label': app_label,
                            'app_url': reverse('admin:app_list', kwargs={'app_label': app_label}, current_app=self.name),
                            'has_module_perms': has_module_perms,
                            'models': [model_dict],
                        }

        # Sort the apps alphabetically.
        app_list = list(six.itervalues(app_dict))
        app_list.sort(key=lambda x: x['name'])

        # Sort the models alphabetically within each app.
        for app in app_list:
            app['models'].sort(key=lambda x: x['name'])

        context = {
            'title': _('Site administration'),
            'app_list': app_list,
        }
        context.update(extra_context or {})
        return TemplateResponse(request, self.index_template or
                                'admin/index.html', context,
                                current_app=self.name)


    def app_index(self, request, app_label, extra_context=None):
        user = request.user
        has_module_perms = user.has_module_perms(app_label)
        app_dict = {}
        for model, model_admin in self._registry.items():
            if app_label == model._meta.app_label:
                if has_module_perms:
                    perms = model_admin.get_model_perms(request)

                    # Check whether user has any perm for this module.
                    # If so, add the module to the model_list.
                    if True in perms.values():
                        info = (app_label, model._meta.model_name)
                        model_dict = {
                            'name': capfirst(model._meta.verbose_name_plural),
                            'object_name': model._meta.object_name,
                            'perms': perms,
                        }
                        if perms.get('change', False):
                            try:
                                model_dict['admin_url'] = reverse('admin:admin:%s_%s_changelist' % info, current_app=self.name)
                            except NoReverseMatch:
                                pass
                        if perms.get('add', False):
                            try:
                                model_dict['add_url'] = reverse('admin:admin:%s_%s_add' % info, current_app=self.name)
                            except NoReverseMatch:
                                pass
                        if app_dict:
                            app_dict['models'].append(model_dict),
                        else:
                            # First time around, now that we know there's
                            # something to display, add in the necessary meta
                            # information.
                            app_dict = {
                                'name': app_label.title(),
                                'app_label': app_label,
                                'app_url': '',
                                'has_module_perms': has_module_perms,
                                'models': [model_dict],
                            }
        if not app_dict:
            raise Http404('The requested admin page does not exist.')
        # Sort the models alphabetically within each app.
        app_dict['models'].sort(key=lambda x: x['name'])
        context = {
            'title': _('%s administration') % capfirst(app_label),
            'app_list': [app_dict],
        }
        context.update(extra_context or {})

        return TemplateResponse(request, self.app_index_template or [
            'admin/%s/app_index.html' % app_label,
            'admin/app_index.html'
        ], context, current_app=self.name)


django.contrib.admin.site = CollectionAdminSite()
site = django.contrib.admin.site


from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.utils.encoding import smart_text
from rest_framework.fields import is_simple_callable, get_component
from rest_framework.relations import PrimaryKeyRelatedField


class RelatedCollectionModelField(PrimaryKeyRelatedField):
    """
    Represents a relationship as a id value.
    """

    # TODO: Remove these field hacks...
    def prepare_value(self, obj):
        return self.to_native(self._get_object_id(obj=obj))

    def label_from_instance(self, obj):
        """
        Return a readable representation for use with eg. select widgets.
        """
        desc = smart_text(obj)
        ident = smart_text(self.to_native(self._get_object_id(obj=obj)))
        if desc == ident:
            return desc
        return "%s - %s" % (desc, ident)

    # TODO: Possibly change this to just take `obj`, through prob less performant
    def to_native(self, id):
        return id

    def from_native(self, data):
        if self.queryset is None:
            raise Exception('Writable related fields must include a `queryset` argument')

        try:
            single_object = self.queryset.get(_id=data)

            # Check if object doesn't exists
            if single_object is None:
                raise ObjectDoesNotExist()
            # Else return it
            else:
                return single_object

        except ObjectDoesNotExist:
            msg = self.error_messages['does_not_exist'] % smart_text(data)
            raise ValidationError(msg)
        except (TypeError, ValueError):
            received = type(data).__name__
            msg = self.error_messages['incorrect_type'] % received
            raise ValidationError(msg)

    def field_to_native(self, obj, field_name):
        if self.many:
            # To-many relationship

            queryset = None
            if not self.source:
                # Prefer obj.serializable_value for performance reasons
                try:
                    queryset = obj.serializable_value(field_name)
                except AttributeError:
                    pass
            if queryset is None:
                # RelatedManager (reverse relationship)
                source = self.source or field_name
                queryset = obj
                for component in source.split('.'):
                    if queryset is None:
                        return []
                    queryset = get_component(queryset, component)

            # Forward relationship
            if is_simple_callable(getattr(queryset, 'all', None)):
                queryset.all()
                return [self.to_native(self._get_object_id(obj=item)) for item in queryset ]
            else:
                # Also support non-queryset iterables.
                # This allows us to also support plain lists of related items.
                return [self.to_native(self._get_object_id(obj=item)) for item in queryset]

        # To-one relationship
        try:
            # Prefer obj.serializable_value for performance reasons
            id = obj.serializable_value(self.source or field_name)
        except AttributeError:
            # RelatedObject (reverse relationship)
            try:
                id = getattr(obj, self.source or field_name).id
            except (ObjectDoesNotExist, AttributeError):
                return None

        # Forward relationship
        return self.to_native(id)

    def _get_object_id(self, obj):
        """
        """

        return obj.document.id


from django.db import models

from arangodb.orm.fields import ModelField
from arangodb.orm.models import CollectionModel, CollectionModelManager, CollectionQueryset
from arangodb.query.advanced import Query


class DjangoQuery(Query):
    """
    """

    def __init__(self):

        super(DjangoQuery, self).__init__()
        self.select_related = True


    def sort_by(self, field, order=None, collection=None):
        """
        """

        if field == 'pk':
            field = '_key'

        if order is None:
            order = self.SORTING_ASC

        self.sorting.append({
            'field': field,
            'order': order,
            'collection': collection,
        })

    def __getattribute__(self, item):
        """
        """

        if item == 'order_by':
            return object.__getattribute__(self, 'sorting')
        else:
            return super(DjangoQuery, self).__getattribute__(item)

class DjangoQueryset(CollectionQueryset):
    """
    """

    def __init__(self, manager):
        """
        """

        super(DjangoQueryset, self).__init__(manager)

        self._query = DjangoQuery()
        self.query = self._query
        
        self.model = manager.model


    def order_by(self, *args):
        """
        """

        # We want only django sorting
        for o in args:
            if o.startswith('-'):
                order = self._query.SORTING_ASC
                o = o.replace('-', '')
            else:
                order = self._query.SORTING_DESC

            field = o

            self._query.sort_by(field=field, order=order)

        self._has_cache = False

        return self


class DjangoModelManager(CollectionModelManager):


    queryset = DjangoQueryset

    def __init__(self, cls):
        """
        """

        super(DjangoModelManager, self).__init__(cls)

        self.model = cls

    def get(self, **kwargs):
        """
        """

        if 'pk' in kwargs:
            search_value = kwargs['pk']
            del kwargs['pk']
            kwargs['_key'] = search_value

        return super(DjangoModelManager, self).get(**kwargs)


class DjangoModel(CollectionModel):

    objects = DjangoModelManager

    @classmethod
    def extend_meta_data(cls, model_class, class_meta):
        """
        """

        def get_field_by_name(field_name):
            """
            """

            fields = model_class.get_all_fields()
            if field_name in fields:
                return [fields[field_name], model_class, False, False]
            else:
                raise models.FieldDoesNotExist

        class_meta.get_field_by_name = get_field_by_name

        # Get field, by name
        class_meta.get_field = lambda field: get_field_by_name(field)[0]

        # PK field
        pk_model_field = ModelField()
        pk_model_field.on_init(model_class, 'pk')

        class_meta.pk = pk_model_field

        class_meta.related_fkey_lookups = []


import os
from datetime import datetime, date

from django.core.files.base import File
from django.core.files.storage import default_storage
from django.conf import settings
from django.utils.encoding import force_str, force_text
from django.utils import timezone
from django.utils.translation import ugettext as _
from django.db.models.fields.files import FileDescriptor

from arangodb.orm.fields import TextField, BooleanField, DateField, DatetimeField


class FieldFile(File):
    def __init__(self, instance, field, name):
        super(FieldFile, self).__init__(None, name)
        self.instance = instance
        self.field = field
        self.storage = field.storage
        self._committed = True

    def __eq__(self, other):
        # Older code may be expecting FileField values to be simple strings.
        # By overriding the == operator, it can remain backwards compatibility.
        if hasattr(other, 'name'):
            return self.name == other.name
        return self.name == other

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(self.name)

    # The standard File contains most of the necessary properties, but
    # FieldFiles can be instantiated without a name, so that needs to
    # be checked for here.

    def _require_file(self):
        if not self:
            raise ValueError("The '%s' attribute has no file associated with it." % self.field.name)

    def _get_file(self):
        self._require_file()
        if not hasattr(self, '_file') or self._file is None:
            self._file = self.storage.open(self.name, 'rb')
        return self._file

    def _set_file(self, file):
        self._file = file

    def _del_file(self):
        del self._file

    file = property(_get_file, _set_file, _del_file)

    def _get_path(self):
        self._require_file()
        return self.storage.path(self.name)
    path = property(_get_path)

    def _get_url(self):
        self._require_file()
        return self.storage.url(self.name)
    url = property(_get_url)

    def _get_size(self):
        self._require_file()
        if not self._committed:
            return self.file.size
        return self.storage.size(self.name)
    size = property(_get_size)

    def open(self, mode='rb'):
        self._require_file()
        self.file.open(mode)
    # open() doesn't alter the file's contents, but it does reset the pointer
    open.alters_data = True

    # In addition to the standard File API, FieldFiles have extra methods
    # to further manipulate the underlying file, as well as update the
    # associated model instance.

    def save(self, name, content, save=True):
        # name = self.field.generate_filename(self.instance, name)
        self.name = self.storage.save(name, content)
        setattr(self.instance, self.field.name, self.name)

        # Update the filesize cache
        self._size = content.size
        self._committed = True

        # Save the object because it has changed, unless save is False
        if save:
            self.instance.save()
    save.alters_data = True

    def delete(self, save=True):
        if not self:
            return
        # Only close the file if it's already open, which we know by the
        # presence of self._file
        if hasattr(self, '_file'):
            self.close()
            del self.file

        self.storage.delete(self.name)

        self.name = None
        setattr(self.instance, self.field.name, self.name)

        # Delete the filesize cache
        if hasattr(self, '_size'):
            del self._size
        self._committed = False

        if save:
            self.instance.save()
    delete.alters_data = True

    def _get_closed(self):
        file = getattr(self, '_file', None)
        return file is None or file.closed
    closed = property(_get_closed)

    def close(self):
        file = getattr(self, '_file', None)
        if file is not None:
            file.close()

    def __getstate__(self):
        # FieldFile needs access to its associated model field and an instance
        # it's attached to in order to work properly, but the only necessary
        # data to be pickled is the file's name itself. Everything else will
        # be restored later, by FileDescriptor below.
        return {'name': self.name, 'closed': False, '_committed': True, '_file': None}


class FileField(TextField):

    # The class to wrap instance attributes in. Accessing the file object off
    # the instance will always return an instance of attr_class.
    attr_cls = FieldFile

    # The descriptor to use for accessing the attribute off of the class.
    descriptor_class = FileDescriptor

    def __init__(self, upload_to='', storage=None, **kwargs):
        """
        """

        super(FileField, self).__init__(**kwargs)

        self.file_name = None

        self.storage = storage or default_storage
        self.upload_to = upload_to
        if callable(upload_to):
            self.generate_filename = upload_to


    def create_file_instance(self, model_instance):
        """
        """

        self.file = self.attr_cls(instance=model_instance, field=self, name=self.file_name)

    def on_save(self, model_instance):
        """
        """

        self.create_file_instance(model_instance)

        file = self.file

        # This was reset
        if self.file_name == self.content_file:
            return

        if file:
            # Commit the file to storage prior to saving the model
            file.save(file.name, self.content_file, save=False)



    def get(self):
        """
        """

        return self.file_name


    def set(self, *args, **kwargs):
        """
        """

        if len(args) is 1:
            value = args[0]

            self.content_file = value

            if not '/' in value:
                self.file_name = self.generate_filename(self, value)
            else:
                self.file_name = value

    def dumps(self):
        """
        """

        return self.file.path


    def loads(self, string_val):
        """
        """

        self.file_name = string_val


    def validate(self):
        """
        """

        super(FileField, self).validate()


    def get_directory_name(self):
        return os.path.normpath(force_text(datetime.now().strftime(force_str(self.upload_to))))

    def get_filename(self, filename):
        return os.path.normpath(self.storage.get_valid_name((filename)))

    def generate_filename(self, instance, filename):
        return os.path.join(self.get_directory_name(), self.get_filename(filename))


class DjangoBooleanField(BooleanField):


    flatchoices = (
        (False, _('False')),
        (True, _('True')),
    )

    def __getattribute__(self, item):
        """
        """

        if item == 'choices':
            return True
        else:
            return super(BooleanField, self).__getattribute__(item)


class DjangoDateField(DateField):
    """
        Can be timezone aware
    """

    def __init__(self, **kwargs):
        """
        """

        super(DjangoDateField, self).__init__(**kwargs)

        if self.null and not self.default:
            self.date = None
        else:
            if self.default:
                self.date = self.default
            else:
                if getattr(settings, 'USE_TZ', False):
                    self.date = timezone.now().date()
                else:
                    self.date = date.today()


class DjangoTimeField(DatetimeField):
    """
        Can be timezone aware
    """

    def __init__(self, **kwargs):
        """
        """

        super(DjangoTimeField, self).__init__(**kwargs)

        if self.null and not self.default:
            self.time = None
        else:
            if self.default:
                self.time = self.default
            else:
                if getattr(settings, 'USE_TZ', False):
                    self.time = timezone.now()
                else:
                    self.time = datetime.now()


# The views used below are normally mapped in django.contrib.admin.urls.py
# This URLs file is used to provide a reliable view deployment for test purposes.
# It is also provided as a convenience to those who want to deploy these URLs
# elsewhere.

from django.contrib import admin
from django.conf.urls import patterns, url, include

urlpatterns = patterns('djara.django.contrib.admin.views',

    url(r'^login/$', 'login', name='login'),
    url(r'^logout/$', 'logout', name='logout'),
    url(r'^(?P<app_label>\w+)/$', admin.site.index, name='app_list'),
    url(r'^jsi18n/$', admin.site.i18n_javascript, name='jsi18n'),
    url(r'^$', admin.site.index, name='index'),
    # url(r'^password_change/$', 'django.contrib.auth.views.password_change', name='password_change'),
    # url(r'^password_change/done/$', 'django.contrib.auth.views.password_change_done', name='password_change_done'),
    # url(r'^password_reset/$', 'django.contrib.auth.views.password_reset', name='password_reset'),
    # url(r'^password_reset/done/$', 'django.contrib.auth.views.password_reset_done', name='password_reset_done'),
    # # Support old style base36 password reset links; remove in Django 1.7
    # url(r'^reset/(?P<uidb36>[0-9A-Za-z]{1,13})-(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
    #     'django.contrib.auth.views.password_reset_confirm_uidb36'),
    # url(r'^reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
    #     'django.contrib.auth.views.password_reset_confirm',
    #     name='password_reset_confirm'),
    # url(r'^reset/done/$', 'django.contrib.auth.views.password_reset_complete', name='password_reset_complete'),
)

# Add the real url patterns of the admin
urlpatterns += admin.site.get_urls()


from django.conf import settings

from rest_framework.fields import FileField


class RestFileField(FileField):

    def to_native(self, value):
        return '%s%s' % (settings.MEDIA_URL,  value.split(settings.MEDIA_URL)[1])


import six
import warnings

from django.core.exceptions import FieldError
from django.forms.util import ErrorList
from django.forms.widgets import media_property
from django.utils.datastructures import SortedDict
from django.forms.fields import CharField, FileField, ChoiceField, BooleanField, DateField, DateTimeField
from django.forms.forms import BaseForm, get_declared_fields
from django.forms.models import ModelFormOptions, ALL_FIELDS, InlineForeignKeyField

from arangodb.orm.fields import CharField as ArangoCharField, ChoiceField as ArangoChoiceField, \
    BooleanField as ArangoBooleanField, ForeignKeyField as ArangoForeignKeyField, DateField as ArangoDateField, \
    DatetimeField as ArangoDatetimeField, ListField, DictField

from djara.django.models.fields import FileField as ArangoFileField


def map_arangopy_field_to_form_field(model_field, **kwargs):
    """
    """

    if isinstance(model_field, ArangoCharField):
        field = CharField(max_length=model_field.max_length, **kwargs)
    elif isinstance(model_field, ListField):
        field = CharField(**kwargs)
    elif isinstance(model_field, DictField):
        field = CharField(**kwargs)
    elif isinstance(model_field, ArangoFileField):
        field = FileField(**kwargs)
    elif isinstance(model_field, ArangoChoiceField):
        field = ChoiceField(choices=model_field.choices)
    elif isinstance(model_field, ArangoBooleanField):
        field = BooleanField(**kwargs)
    elif isinstance(model_field, ArangoDateField):
        field = DateField(**kwargs)
    elif isinstance(model_field, ArangoDatetimeField):
        field = DateTimeField(**kwargs)
    elif isinstance(model_field, ArangoForeignKeyField):
        field = InlineForeignKeyField(**kwargs)
    else:
        field = None

    return field


def fields_for_model(model, fields=None, exclude=None, widgets=None,
                     formfield_callback=None, localized_fields=None,
                     labels=None, help_texts=None, error_messages=None):
    """
    Returns a ``SortedDict`` containing form fields for the given model.

    ``fields`` is an optional list of field names. If provided, only the named
    fields will be included in the returned fields.

    ``exclude`` is an optional list of field names. If provided, the named
    fields will be excluded from the returned fields, even if they are listed
    in the ``fields`` argument.

    ``widgets`` is a dictionary of model field names mapped to a widget.

    ``localized_fields`` is a list of names of fields which should be localized.

    ``labels`` is a dictionary of model field names mapped to a label.

    ``help_texts`` is a dictionary of model field names mapped to a help text.

    ``error_messages`` is a dictionary of model field names mapped to a
    dictionary of error messages.

    ``formfield_callback`` is a callable that takes a model field and returns
    a form field.
    """
    field_list = []
    ignored = []
    # Avoid circular import
    # from django.db.models.fields import Field as ModelField
    # sortable_virtual_fields = [f for f in opts.virtual_fields
    #                            if isinstance(f, ModelField)]
    sortable_virtual_fields = model.get_collection_fields()
    for f in sorted(sortable_virtual_fields):
        if getattr(f, 'read_only', True):
            continue
        if fields is not None and not f.name in fields:
            continue
        if exclude and f.name in exclude:
            continue

        kwargs = {}
        if widgets and f.name in widgets:
            kwargs['widget'] = widgets[f.name]
        if localized_fields == ALL_FIELDS or (localized_fields and f.name in localized_fields):
            kwargs['localize'] = True
        if labels and f.name in labels:
            kwargs['label'] = labels[f.name]
        if help_texts and f.name in help_texts:
            kwargs['help_text'] = help_texts[f.name]
        if error_messages and f.name in error_messages:
            kwargs['error_messages'] = error_messages[f.name]

        if formfield_callback is None:
            formfield = map_arangopy_field_to_form_field(f, **kwargs)
            # formfield = f.formfield(**kwargs)
        elif not callable(formfield_callback):
            raise TypeError('formfield_callback must be a function or callable')
        else:
            formfield = formfield_callback(f, **kwargs)

        if formfield or fields is None:
            field_list.append((f.name, formfield))
        else:
            ignored.append(f.name)
    field_dict = SortedDict(field_list)

    if fields:
        field_dict = SortedDict(
            [(f, field_dict.get(f)) for f in fields
                if ((not exclude) or (exclude and f not in exclude)) and (f not in ignored)]
        )
    return field_dict

class BaseCollectionForm(BaseForm):

    def __init__(self, data=None, files=None, auto_id='id_%s', prefix=None,
                 initial=None, error_class=ErrorList, label_suffix=None,
                 empty_permitted=False, instance=None):
        opts = self._meta
        if opts.model is None:
            raise ValueError('ModelForm has no model class specified.')
        if instance is None:
            # if we didn't get an instance, instantiate a new one
            self.instance = opts.model()
            object_data = {}
        else:
            self.instance = instance
            object_data = {}

            exclude = []
            if hasattr(opts, 'exclude'):
                exclude = exclude

            for field_name in opts.fields:
                if field_name in exclude:
                    continue

                object_data[field_name] = getattr(instance, field_name)

        # if initial was provided, it should override the values from instance
        if initial is not None:
            object_data.update(initial)
        # self._validate_unique will be set to True by BaseModelForm.clean().
        # It is False by default so overriding self.clean() and failing to call
        # super will stop validate_unique from being called.
        self._validate_unique = False
        super(BaseCollectionForm, self).__init__(data, files, auto_id, prefix, object_data,
                                            error_class, label_suffix, empty_permitted)

class CollectionFormMetaclass(type):
    def __new__(cls, name, bases, attrs):
        formfield_callback = attrs.pop('formfield_callback', None)
        try:
            parents = [b for b in bases if issubclass(b, CollectionForm)]
        except NameError:
            # We are defining ModelForm itself.
            parents = None

        declared_fields = get_declared_fields(bases, attrs, False)
        new_class = super(CollectionFormMetaclass, cls).__new__(cls, name, bases,
                attrs)
        if not parents:
            return new_class

        if 'media' not in attrs:
            new_class.media = media_property(new_class)
        opts = new_class._meta = ModelFormOptions(getattr(new_class, 'Meta', None))

        # We check if a string was passed to `fields` or `exclude`,
        # which is likely to be a mistake where the user typed ('foo') instead
        # of ('foo',)
        for opt in ['fields', 'exclude', 'localized_fields']:
            value = getattr(opts, opt)
            if isinstance(value, six.string_types) and value != ALL_FIELDS:
                msg = ("%(model)s.Meta.%(opt)s cannot be a string. "
                       "Did you mean to type: ('%(value)s',)?" % {
                           'model': new_class.__name__,
                           'opt': opt,
                           'value': value,
                       })
                raise TypeError(msg)

        if opts.model:
            # If a model is defined, extract form fields from it.

            if opts.fields is None and opts.exclude is None:
                # This should be some kind of assertion error once deprecation
                # cycle is complete.
                warnings.warn("Creating a ModelForm without either the 'fields' attribute "
                              "or the 'exclude' attribute is deprecated - form %s "
                              "needs updating" % name,
                              PendingDeprecationWarning, stacklevel=2)

            if opts.fields == ALL_FIELDS:
                # sentinel for fields_for_model to indicate "get the list of
                # fields from the model"
                opts.fields = None

            fields = fields_for_model(opts.model, opts.fields, opts.exclude,
                                      opts.widgets, formfield_callback,
                                      opts.localized_fields, opts.labels,
                                      opts.help_texts, opts.error_messages)

            # make sure opts.fields doesn't specify an invalid field
            none_model_fields = [k for k, v in six.iteritems(fields) if not v]
            missing_fields = set(none_model_fields) - \
                             set(declared_fields.keys())
            if missing_fields:
                message = 'Unknown field(s) (%s) specified for %s'
                message = message % (', '.join(missing_fields),
                                     opts.model.__name__)
                raise FieldError(message)
            # Override default model fields with any custom declared ones
            # (plus, include all the other declared fields).
            fields.update(declared_fields)
        else:
            fields = declared_fields
        new_class.declared_fields = declared_fields
        new_class.base_fields = fields
        return new_class

class CollectionForm(six.with_metaclass(CollectionFormMetaclass, BaseCollectionForm)):

    def save(self, commit=True):
        """
        """

        for key, value in self.cleaned_data.items():
            setattr(self.instance, key, value)

        self.instance.save()


from django.middleware.csrf import rotate_token

from djara.django.auth.models import User

SESSION_KEY = '_auth_user_id'

def get_session_user(request):
    """
    Returns the user model instance associated with the given request session.
    If no user is retrieved an instance of `AnonymousUser` is returned.
    """
    from .models import AnonymousUser
    try:
        user_id = request.session[SESSION_KEY]
        user = User.objects.get(_id=user_id) or AnonymousUser()
    except (KeyError, AssertionError):
        user = AnonymousUser()

    return user


def login(request, user):

    if user is None:
        user = request.user

    # TODO: It would be nice to support different login methods, like signed cookies.
    if SESSION_KEY in request.session:
        if request.session[SESSION_KEY] != user.id:
            # To avoid reusing another user's session, create a new, empty
            # session if the existing session corresponds to a different
            # authenticated user.
            request.session.flush()
    else:
        request.session.cycle_key()

    request.session[SESSION_KEY] = user.id

    if hasattr(request, 'user'):
        request.user = user

    rotate_token(request)

def logout(request):
    """
    Removes the authenticated user's ID from the request and flushes their
    session data.
    """
    # Dispatch the signal before the user is logged out so the receivers have a
    # chance to find out *who* logged out.
    user = getattr(request, 'user', None)

    # Check if the user has ever been logged out
    if hasattr(user, 'is_authenticated') and not user.is_authenticated():
        return False

    # remember language choice saved to session
    language = request.session.get('django_language')

    request.session.flush()

    if language is not None:
        request.session['django_language'] = language

    if hasattr(request, 'user'):
        from .models import AnonymousUser
        request.user = AnonymousUser()

    return True

def authenticate(username, password):
    """
    """

    user = User.objects.get(username=username)
    if not user.password.verify(password):
        return None
    else:
        return user


from setuptools import setup, find_packages

setup(
    name='ArangoDjango',
    version= '0.0.25',
    packages=find_packages(),
    include_package_data=True,
    requires=[
        'Django',
        'djangorestframework',
        'ArangoPy',
    ],
    url='https://github.com/saeschdivara/ArangoDjango',
    license='MIT',
    author='saskyrardisaskyr',
    author_email='saeschdivara@gmail.com',
    description='Bridge between the django and the arangodb world'
)



from django.conf import settings
from django.conf.urls import patterns, include, url
from django.conf.urls.i18n import i18n_patterns

###############
## I18N URLS ##
###############
urlpatterns = patterns('djara.django.auth.views',

    # Login
    url(r'^login/$', view='login_view', name='login-view'),
    url(r'^login/action/(?P<view_name>[a-z|A-Z]+[:][a-z|A-Z]+)/$', view='login_action', name='login-action'),

    # Logout
    url(r'^logout/action/(?P<view_name>[a-z|A-Z]+[:][a-z|A-Z]+)/$', view='logout_action', name='logout-action'),

    # Register
    url(r'^register/$', view='register_view', name='register-view'),
    url(r'^register/action/(?P<view_name>[a-z|A-Z]+[:][a-z|A-Z]+)/$', view='register_action', name='register-action'),
    url(r'^register/activate/(?P<activation_key>.+)/(?P<uuid>.+)/(?P<view_name>[a-z|A-Z]+[:][a-z|A-Z]+)/$', view='register_activate', name='register-activate'),
)



from django.core.mail import get_connection
from django.core.mail.message import EmailMessage


def send_html_email(subject, message, from_email, recipient_list,
              fail_silently=False, auth_user=None, auth_password=None,
              connection=None):
    """
    Easy wrapper for sending a single message to a recipient list. All members
    of the recipient list will see the other recipients in the 'To' field.

    If auth_user is None, the EMAIL_HOST_USER setting is used.
    If auth_password is None, the EMAIL_HOST_PASSWORD setting is used.

    Note: The API for this method is frozen. New code wanting to extend the
    functionality should use the EmailMessage class directly.
    """
    connection = connection or get_connection(username=auth_user,
                                    password=auth_password,
                                    fail_silently=fail_silently)

    msg = EmailMessage(subject, message, from_email, recipient_list, connection=connection)
    msg.content_subtype = 'html'
    return msg.send()